package org.zeroturnaround.javarebel.integration.servlet.jasper;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>org.apache.jasper.compiler.Compiler</code>.
 * 
 * @author Rein Raudjärv
 * 
 * @see JspServletCBP
 */
public class CompilerCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws CannotCompileException {
    try {
      cp.importPackage("java.net");
      cp.importPackage("org.zeroturnaround.javarebel");
      cp.importPackage("org.zeroturnaround.javarebel.integration.servlet.jasper");
      
      CtMethod m = ctClass.getDeclaredMethod("isOutDated", cp.get(new String[] {"boolean"}));
      m.insertBefore("{" +
      		"  String jsp = null;" +
      		"  boolean jspRemoved = false;" +
      		
      		"  try {" +
      		"    jsp = ctxt.getJspFile();" +
      		"    URL jspUrl = ctxt.getResource(jsp);" +
      		
      		"    if (jspUrl == null) {" +
      		// Remove the page immediately - otherwise it would be reflected from the second try 
      		"      ctxt.incrementRemoved();" +
      		"      removeGeneratedFiles();" +

      		"      JspCompilationContextExtra extra = (JspCompilationContextExtra) ctxt;" +
      		"      org.apache.jasper.compiler.JspRuntimeContext rctxt = extra.__getRuntimeContext();" +
      		"      if (rctxt != null)" +
      		"        ((org.apache.jasper.compiler.JspRuntimeContext) rctxt).removeWrapper(jsp);" +
      		"      jspRemoved = true;" +
      		"    }" +
            "  } catch (Throwable t) {" +
            "    LoggerFactory.getInstance().error(t);" +
      		"  }" +

      		"  if (jspRemoved) {" +
      		"    throw new " + JspRemovedException.class.getName() + "(jsp);" +
      		"  }" +
      		"}");
    } catch (NotFoundException e) {
    }
  }
}
