package org.zeroturnaround.javarebel.integration.servlet.resin;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>com.caucho.jsp.Page</code>
 * to support JSP pages provided by JavaRebel.
 * 
 * <p>Tested on Resin 3.0.27, 3.1.3 and 3.2.1.</p> 
 * 
 * @author Rein Raudjärv
 */
public class PageCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("java.io");
    cp.importPackage("java.util");
    cp.importPackage("java.lang.reflect");
    cp.importPackage("com.caucho.util");
    cp.importPackage("com.caucho.vfs");
    cp.importPackage("org.zeroturnaround.javarebel");
    
    try {
      ctClass.getField("_lastUpdateCheck");
    } catch (NotFoundException e) {
      // This field does not exist in Resin 4.0
      // TODO implement JSP support for Resin 4.0
      return;
    }
    
    /*
     * _application in Resin 3.0
     * _webApp in Resin 3.1+
     */
    String servletContextField;
    try {
      ctClass.getDeclaredField("_webApp");
      servletContextField = "_webApp";
    } catch (NotFoundException e) {
      servletContextField = "_application";
    }
    
    ctClass.addMethod(CtNewMethod.make(
        "private void _rebel_cleanClassDir() throws Exception {" +
        "  String classDir = _jspManager.getClassDir().getFullPath();" +
        
        "  Method m = this.getClass().getDeclaredMethod(\"_caucho_getDependList\", new Class[0]);" +
        "  ArrayList _caucho_depends = (ArrayList) m.invoke(this, null);" +
        
        "  for (Iterator it = _caucho_depends.iterator(); it.hasNext();) {" +
        "    Depend dp = (Depend) it.next();" +
        "    Path path = dp.getPath();" +
        "    String fullPath = path.getFullPath();" +
        
        "    if (path.exists() && fullPath.startsWith(classDir)) {" +
        "      LoggerFactory.getInstance().log(\"Deleting path: \" + fullPath);" +
        "      path.remove();" +
        "    }" +
        "  }" +
        "}",
        ctClass));
    
    ctClass.addMethod(CtNewMethod.make(
        "private boolean _rebel_isModified() throws Exception {" +
        "  String name = getServletName();" +
        "  RebelServletContext servletContext = (RebelServletContext) " + servletContextField + ";" +
        "  ServletIntegration integration = ServletIntegrationFactory.getInstance();" +
        "  if (!integration.isResourceReplaced(servletContext, name))" +
        "    return false;" +

        "  String realPath = integration.getRealPath(servletContext, name);" +
        "  if (realPath == null)" +
        "    return false;" +

        "  long lastModified = new File(realPath).lastModified();" +
        "  boolean isModified = lastModified > _caucho_lastModified();" +
        
        "  if (isModified)" +
        "    _rebel_cleanClassDir();" +

        "  return isModified;" +
        "}",
        ctClass));
    
    try {
      CtMethod m = ctClass.getDeclaredMethod("cauchoIsModified");
      m.insertBefore(
          "{" +
          "  try {" +
          "    if (_rebel_isModified()) {" +
          "      _lastUpdateCheck = Alarm.getCurrentTime();" +
          "      LoggerFactory.getInstance().log(\"JSP page '\" + getServletName() + \"' update detected.\");" +
          "      return true;" +
          "    }" +
          "  } catch (Throwable t) {" +
          "    LoggerFactory.getInstance().error(t);" +
          "  }" +
          "}");
    } catch (NotFoundException e) {
      // skip
    }
  }
}
